//***********************************************************************//
//																		 //
//		- "Talk to me like I'm a 3 year old!" Programming Lessons -		 //
//                                                                       //
//		$Author:		DigiBen		DigiBen@GameTutorials.com			 //
//																		 //
//		$Program:		Camera4											 //
//																		 //
//		$Description:	Rotates the camera in 3rd person mode			 //
//																		 //
//		$Date:			11/1/01											 //
//																		 //
//***********************************************************************//

#include "main.h"


///////////////////////////////// CCAMERA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This is the class constructor
/////
///////////////////////////////// CCAMERA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

CCamera::CCamera()
{
	CVector vZero = {0.0, 0.0, 0.0};		// Init a vector to 0 0 0 for our position
	CVector vView = {0.0, 1.0, 0.5};		// Init a starting view vector (looking up and out the screen) 
	CVector vUp   = {0.0, 0.0, 1.0};		// Init a standard up vector (Rarely ever changes)

	mPosition	= vZero;					// Init the position to zero
	mView		= vView;					// Init the view to a std starting view
	mUpVector	= vUp;						// Init the UpVector
}


///////////////////////////////// POSITION CAMERA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This function sets the camera's position and view and up vector.
/////
///////////////////////////////// POSITION CAMERA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

GLvoid CCamera::PositionCamera(float positionX, float positionY, float positionZ,
					  		   float viewX,     float viewY,     float viewZ,
							   float upVectorX, float upVectorY, float upVectorZ)
{
	CVector vPosition	= {positionX, positionY, positionZ};
	CVector vView		= {viewX, viewY, viewZ};
	CVector vUpVector	= {upVectorX, upVectorY, upVectorZ};

	// The code above just makes it cleaner to set the variables.
	// Otherwise we would have to set each variable x y and z.

	mPosition = vPosition;					// Assign the position
	mView = vView;							// Assign the view
	mUpVector = vUpVector;					// Assign the up vector
}


///////////////////////////////// ROTATE VIEW \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This rotates the view around the position
/////
///////////////////////////////// ROTATE VIEW \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void CCamera::RotateView(float X, float Y, float Z)
{
	CVector vector;							// Vector for the position/view.

	// Get our view vector (The direction we are facing)
	vector.x = mView.x - mPosition.x;		// This gets the direction of the X	
	vector.y = mView.y - mPosition.y;		// This gets the direction of the Y
	vector.z = mView.z - mPosition.z;		// This gets the direction of the Z

	// If we pass in a negative X Y or Z, it will rotate the opposite way,
	// so we only need one function for a left and right , up or down rotation.
	// I suppose we could have one move function too, but I decided not too.

	if(X) {
		mView.z = (float)(mPosition.z + sin(X)*vector.y + cos(X)*vector.z);
		mView.y = (float)(mPosition.y + cos(X)*vector.y - sin(X)*vector.z);
	}
	if(Y) {
		mView.z = (float)(mPosition.z + sin(Y)*vector.x + cos(Y)*vector.z);
		mView.x = (float)(mPosition.x + cos(Y)*vector.x - sin(Y)*vector.z);
	}
	if(Z) {
		mView.x = (float)(mPosition.x + sin(Z)*vector.y + cos(Z)*vector.x);		
		mView.y = (float)(mPosition.y + cos(Z)*vector.y - sin(Z)*vector.x);
	}
}


//////////// *** NEW *** ////////// *** NEW *** ///////////// *** NEW *** ////////////////////

///////////////////////////////// ROTATE AROUND POINT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This rotates the position around a given point
/////
///////////////////////////////// ROTATE AROUND POINT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void CCamera::RotateAroundPoint(CVector vCenter, float X, float Y, float Z)
{
	CVector vector;							// Vector for the position/view.

	// To rotate our position around a point, what we need to do is find
	// a vector from our position to the center point we will be rotating around.
	// Once we get this vector then we rotate it along the specified axis with
	// the specified degree.  After that we add it to the center point that we
	// rotated around (vCenter).  That's all it takes.

	// Get the vector from our position to the center we are rotating around
	vector.x = mPosition.x - vCenter.x;		// This gets the direction of the X	
	vector.y = mPosition.y - vCenter.y;		// This gets the direction of the Y
	vector.z = mPosition.z - vCenter.z;		// This gets the direction of the Z

	// Rotate the position along the desired axis around the desired point vCenter
	if(X) {

		// Rotate the position up or down, then add it to the center point
		mPosition.z = (float)(vCenter.z + sin(X)*vector.y + cos(X)*vector.z);
		mPosition.y = (float)(vCenter.y + cos(X)*vector.y - sin(X)*vector.z);
	}
	if(Y) {
		// Rotate the position right or left, then add it to the center point
		mPosition.z = (float)(vCenter.z + sin(Y)*vector.x + cos(Y)*vector.z);
		mPosition.x = (float)(vCenter.x + cos(Y)*vector.x - sin(Y)*vector.z);
	}
	if(Z) {
		// Rotate the position diagnally right or diagnally down, then add it to the center point
		mPosition.x = (float)(vCenter.x + sin(Z)*vector.y + cos(Z)*vector.x);		
		mPosition.y = (float)(vCenter.y + cos(Z)*vector.y - sin(Z)*vector.x);
	}

	// Now mPosition should be our new rotated position
}

//////////// *** NEW *** ////////// *** NEW *** ///////////// *** NEW *** ////////////////////


///////////////////////////////// MOVE CAMERA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	This will move the camera forward or backward depending on the speed
/////
///////////////////////////////// MOVE CAMERA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void CCamera::MoveCamera(float speed)
{
	CVector vector={0};						// Init a vector for our view

	// Get our view vector (The direciton we are facing)
	vector.x = mView.x - mPosition.x;		// This gets the direction of the X	
	vector.y = mView.y - mPosition.y;		// This gets the direction of the Y
	vector.z = mView.z - mPosition.z;		// This gets the direction of the Z

	mPosition.x += vector.x * speed;		// Add our acceleration to our position's X
	mPosition.z += vector.z * speed;		// Add our acceleration to our position's Z
	mView.x += vector.x * speed;			// Add our acceleration to our view's X
	mView.z += vector.z * speed;			// Add our acceleration to our view's Z
}


/////////////////////////////////////////////////////////////////////////////////
//
// * QUICK NOTES * 
//
// This tutorial demonstrates how to do a third person camera view.  There is a lot
// more that this view deals with, but this is the basics.  How it's done is really
// simple.  Instead of rotating the view around the position, you just rotate the
// position around the view.  I just took RotateView() and swapped the places that
// mView was with mPosition and vice versa.  However, we use vCenter because we
// might not always want to use the view, it could be some other point.  That way
// it's modular and doesn't have to just be rotating around the view.  In this
// tutorial we just pass in gCamera.mView for the point to rotate around.
//
// We added RotateAroundPoint() to our camera class.  This takes in the point to
// rotate the camera position around (We pass in mView) and the desired axis with
// a rotation degree.
//
// Not much else to say about this, so I hope this helps someone out that might
// be struggling with this.  You can use the arrow keys to move around the world.
// 
// Enjoy!
//
// 
// Ben Humphrey (DigiBen)
// Game Programmer
// DigiBen@GameTutorials.com
// Co-Web Host of www.GameTutorials.com
//
//
