#ifndef _MAIN_H
#define _MAIN_H


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gl\gl.h>										// Header File For The OpenGL32 Library
#include <gl\glu.h>										// Header File For The GLu32 Library

#define SCREEN_WIDTH 800								// We want our screen width 800 pixels
#define SCREEN_HEIGHT 600								// We want our screen height 600 pixels
#define SCREEN_DEPTH 16									// We want 16 bits per pixel

// Here is our vector class
class CVector {											// Remember, we use a C in front to show it's a class

public:													// We want this data public so we don't have to use data access functions.  We could use a struct but I chose not to.
	float x, y, z;										// We just want a float for a X Y and Z.
};

// This is our camera class
class CCamera {

public:

	CVector mPosition;									// The camera's position
	CVector mView;										// The camera's View
	CVector mUpVector;									// The camera's UpVector

	// Our camera constructor
	CCamera();	
	
	// This changes the position, view, and up vector of the camera.
	// This is primarily used for initialization
	void PositionCamera(float positionX, float positionY, float positionZ,
			 		    float viewX,     float viewY,     float viewZ,
						float upVectorX, float upVectorY, float upVectorZ);

	// This rotates the camera's view around the position depending on the values passed in.
	void RotateView(float X, float Y, float Z);


//////////// *** NEW *** ////////// *** NEW *** ///////////// *** NEW *** ////////////////////

	// This rotates the camera around a point (I.E. your character).
	// vCenter is the point that we want to rotate the position around.
	// Like the other rotate function, the X, Y and Z are the axis to rotate around.
	void RotateAroundPoint(CVector vCenter, float X, float Y, float Z);

//////////// *** NEW *** ////////// *** NEW *** ///////////// *** NEW *** ////////////////////


	// This will move the camera forward or backward depending on the speed
	void MoveCamera(float speed);
};


// Extern our global variables so other source files can use them
extern CCamera	gCamera;									// This will be our global camera data
extern BOOL		bFullScreen;								// Set full screen as default
extern HWND		ghWnd;										// This is the handle for the window
extern RECT		gRect;										// This holds the window dimensions
extern HDC		ghDC;										// General HDC - (handle to device context)
extern HGLRC	ghRC;										// General OpenGL_DC - Our Rendering Context for OpenGL

// This is our MAIN() for windows
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hprev, PSTR cmdline, int ishow);

// The window proc which handles all of window's messages.
LRESULT CALLBACK WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

// This controls our main program loop
WPARAM MainLoop();

// This changes the screen to full screen mode
void ChangeToFullScreen();

// This is our own function that makes creating a window modular and easy
HWND CreateMyWindow(LPSTR szWindowName, int width, int height, DWORD dwStyle, BOOL fullScreen, HINSTANCE hInstance);

// This allows us to configure our window for OpenGL and backbuffered
BOOL bSetupPixelFormat(HDC hdc);

// This inits our screen translations and projections
void SizeOpenGLScreen(int width, int height);

// This sets up OpenGL
void InitializeOpenGL(int width, int height);

// This initializes the whole program
void Init(HWND hWnd);

// This draws everything to the screen
void RenderScene();

// This frees all our memory in our program
void DeInit();

#endif 


/////////////////////////////////////////////////////////////////////////////////
//
// * QUICK NOTES * 
//
// We added RotateAroundPoint() to our CCamera class.  This allows us to rotate
// our position around a center point. This is used for third person camera views.
//  
// 
// 
// Ben Humphrey (DigiBen)
// Game Programmer
// DigiBen@GameTutorials.com
// Co-Web Host of www.GameTutorials.com
//
//
