//
// Simple diffraction with anisotropic Ward lighting fragment shader
// Code based on chapter 8 from GPU Gems
//

varying vec3 vt;
varying	vec3 lt;
varying	vec3 ht;

uniform sampler2D tangentMap;
uniform sampler2D decalMap;
uniform sampler1D rainbowMap;

void main (void)
{
	const vec4	specColor = vec4 ( 0, 0, 1, 0 );
	const vec3	n         = vec3 ( 0, 0, 1 );
	const float	roughness = 30.0;
	const float	d         = 40.0;

	float	r          = length ( gl_TexCoord [0].xy - vec2 ( 0.5 ) );

	if ( r > 0.5 || r < 0.1 )
		discard;

	vec3	tang       = normalize ( 2.0 * texture2D ( tangentMap, gl_TexCoord [0].xy ).xyz - vec3 ( 1 ) );
	float	u          = dot ( ht, tang );
	float	w          = dot ( ht, n );
	float	e          = roughness * u / w;
	vec3	diffrColor = vec3 ( 0.0 );

	u = d * abs ( u );

	for ( int n = 1; n <= 8; n++ )
		diffrColor += texture1D ( rainbowMap, u / float ( n ) ).rgb;

	gl_FragColor.rgb = specColor.rgb * exp ( -e*e ) + diffrColor * 0.25;
	gl_FragColor.a   = 1.0;
}
