//
// Refractive glass-2 fragment shader
//

uniform samplerCube envMap;

varying vec3  n;
varying vec3  e;

void main (void)
{
	const vec3	eta = vec3 ( 0.9, 0.94, 0.97 );

    					// Compute reflection vector
  	vec3	en = normalize ( e );
  	vec3	nn = normalize ( n );
    vec3	r  = reflect   ( en, nn );

    					// Do a lookup into the environment map.
    vec3 envColor = textureCube ( envMap, r ).rgb;

    					// calc fresnels term.  This allows a view dependant
    					// blend of reflection/refraction
    float fresnel = abs ( dot ( en, nn ) );

    fresnel = clamp ( fresnel, 0.1, 0.9 );

						// calc refraction (transmitted ray) for every component
	vec3 	tRed   = refract ( en, nn, eta.r );
	vec3 	tGreen = refract ( en, nn, eta.g );
	vec3 	tBlue  = refract ( en, nn, eta.b );

    vec3 	refractionColorRed   = textureCube ( envMap, tRed   ).rgb;
    vec3 	refractionColorGreen = textureCube ( envMap, tGreen ).rgb;
    vec3 	refractionColorBlue  = textureCube ( envMap, tBlue  ).rgb;
	vec3	refractionColor      = vec3 ( refractionColorRed.r,
	                                      refractionColorGreen.g,
	                                      refractionColorBlue.b );

    envColor = mix ( envColor, refractionColor, fresnel );

    gl_FragColor = vec4 ( envColor, 1.0 );
}