//
// Example of soap bubbles via GLSL programs
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include    "libExt.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include	<glut.h>
#endif

#include    <stdio.h>
#include    <stdlib.h>

#include    "libTexture.h"
#include    "TypeDefs.h"
#include	"Vector4D.h"
#include    "Vector3D.h"
#include    "Vector2D.h"
#include    "boxes.h"
#include    "GlslProgram.h"

Vector3D    eye   ( 0, 0, 0 );  		// camera position
unsigned    decalMap;                   // decal (diffuse) texture
unsigned    stoneMap;
unsigned    teapotMap;
unsigned	fringeMap;
unsigned	reflectionMap;
float       angle     = 0;
float       rot       = 0;
float       rot2      = 0;
bool        useFilter   = true;
bool		useRotation = true;
int		 	object      = 0;
Vector3D	eta ( 1.05, 1.1, 1.15 );

GlslProgram 	program;

void init ()
{
    glClearColor ( 0.0, 0.0, 0.0, 1.0 );
    glEnable     ( GL_DEPTH_TEST );
    glEnable     ( GL_TEXTURE_2D );
    glDepthFunc  ( GL_LEQUAL     );

    glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
    glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void displayBoxes ()
{
	float	mv [16];

    glMatrixMode ( GL_MODELVIEW );
    glPushMatrix ();

    glActiveTextureARB ( GL_TEXTURE0_ARB );

    glRotatef   ( rot,  0, 0, 1 );
    glRotatef   ( rot2, 1, 0, 0 );
    glGetFloatv ( GL_MODELVIEW_MATRIX,  mv );

    drawCubicMappedBox ( Vector3D ( 4, 4, 4 ), reflectionMap );

    program.bind ();
	program.setUniformMatrix ( "cubeMapMatrix", mv );

	glPopMatrix  ();
    glPushMatrix ();
    glTranslatef ( 0, 2, 0 );

    glRotatef    ( angle * 45.3, 1, 0, 0 );
   	glRotatef    ( angle * 57.2, 0, 1, 0 );

	if ( object == 0 )
		glutSolidTeapot ( 0.6 );
	else
	if ( object == 1 )
		glutSolidSphere ( 0.6, 30, 30 );
	else
	if ( object == 2 )
		glutSolidCube ( 0.8 );
	else
	if ( object == 3 )
		glutSolidTorus ( 0.25, 0.5, 40, 40 );
	else
	if ( object == 4 )
		glutSolidOctahedron ();
	else
	if ( object == 5 )
		glutSolidTetrahedron ();
	else
	if ( object == 6 )
		glutSolidIcosahedron ();

    program.unbind ();

	glDisable   ( GL_BLEND );

    glActiveTextureARB ( GL_TEXTURE1_ARB );
    glBindTexture      ( GL_TEXTURE_CUBE_MAP, 0 );

    glPopMatrix ();
}

void display ()
{
    glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    displayBoxes ();

    glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
   gluLookAt      ( eye.x, eye.y, eye.z,    // eye
                    0, 2, 0,                // center
                    0, 0, 1 );              // up
}

void key ( unsigned char key, int x, int y )
{
    if ( key == 27 || key == 'q' || key == 'Q' )        // quit requested
        exit ( 0 );

    if ( key == ' ' )
    	useRotation = !useRotation;

	if ( key >= '1' && key <= '6' )
		object = 1 + key - '1';
	else
	if ( key == '+' )
	{
		if ( ++object > 6 )
			object = 0;
	}
	else
	if ( key == '-' )
	{
		if ( --object < 0 )
			object = 6;
	}
}

void    specialKey ( int key, int x, int y )
{
    if ( key == GLUT_KEY_RIGHT )
        rot += 5;
    else
    if ( key == GLUT_KEY_LEFT )
        rot -= 5;

    if ( key == GLUT_KEY_DOWN )
        rot2 += 5;
    else
    if ( key == GLUT_KEY_UP )
        rot2 -= 5;

    glutPostRedisplay ();
}

void    animate ()
{
	static	float	lastTime = 0.0;
	float			time     = 0.001f * glutGet ( GLUT_ELAPSED_TIME );

    if ( useRotation )
	    angle += 2 * (time - lastTime);

    lastTime = time;

    program.bind ();
    program.setUniformVector ( "eyePos", Vector4D ( eye, 1 ) );
    program.unbind ();

    glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
                                // initialize glut
    glutInit            ( &argc, argv );
    glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    glutInitWindowSize  ( 512, 512 );


                                // create window
    glutCreateWindow ( "OpenGL refractive glass" );

                                // register handlers
    glutDisplayFunc  ( display    );
    glutReshapeFunc  ( reshape    );
    glutKeyboardFunc ( key        );
    glutSpecialFunc  ( specialKey );
    glutIdleFunc     ( animate    );

    init           ();
    initExtensions ();

	if ( !GlslProgram :: isSupported () )
	{
		printf ( "GLSL not supported.\n" );

		return 1;
	}

    if ( !program.loadShaders ( "glass.vsh", "glass.fsh" ) )
    {
        printf ( "Error loading shaders:\n%s\n", program.getLog ().c_str () );

        return 3;
    }
/*
	const char * faces [6] =
	{
		"../../Textures/Cubemaps/posy.jpg",
		"../../Textures/Cubemaps/posz.jpg",
		"../../Textures/Cubemaps/posx.jpg",
		"../../Textures/Cubemaps/negy.jpg",
		"../../Textures/Cubemaps/negz.jpg",
		"../../Textures/Cubemaps/negx.jpg"
	};
*/
	const char * faces [6] =
	{
		"../../Textures/Cubemaps/cm_left.tga",
		"../../Textures/Cubemaps/cm_right.tga",
		"../../Textures/Cubemaps/cm_bottom.tga",
		"../../Textures/Cubemaps/cm_top.tga",
		"../../Textures/Cubemaps/cm_back.tga",
		"../../Textures/Cubemaps/cm_front.tga",
	};

	reflectionMap = createCubeMap ( true, faces );
//	reflectionMap = createCubeMap ( true, "../../Textures/Cubemaps/Snow.dds" );

	fringeMap = createTexture1D ( true, "fringe.png" );

    decalMap  = createTexture2D ( true, "../../Textures/oak.bmp" );
    stoneMap  = createTexture2D ( true, "../../Textures/block.bmp" );
    teapotMap = createTexture2D ( true, "../../Textures/Oxidated.jpg" );

    program.bind ();
    program.setTexture ( "envMap", 0 );
    program.unbind ();

    printf ( "Space toggles rotation on/offs\n" );

    glutMainLoop ();

    return 0;
}
