//
// HSV correction image filter fragment shader
//

uniform	sampler2D	mainTex;
uniform	float		hScale;
uniform	float		sScale;
uniform	float		vScale;

vec3	rgbToHsv ( const in vec3 c )
{
	float	mn    = min ( min ( c.r, c.g ), c.b );
	float	mx    = max ( max ( c.r, c.g ), c.b );
	float	delta = mx - mn;
	float	h, s, v;

	v = mx;

	if ( mx > 0.001 )
	{
		s = delta / mx;

		if ( c.r == mx )
			h = ( c.g - c.b ) / delta;
		else
		if ( c.g == mx )
			h = 2.0 + ( c.b - c.r ) / delta;
		else
			h = 4.0 + ( c.r - c.g ) / delta;
	}
	else
	{
		s = 0.0;
		h = 0.0;
	}

	return vec3 ( h / 6.0, s, v );
}

vec3	hsvToRgb ( const in vec3 c )
{
	float	r, b, g;
	float	h = c.x;
	float	s = c.y;
	float	v = c.z;
	vec3	res;

	if ( s < 0.001 )
		res = vec3 ( v, v, v );
	else
	{
		h *= 6.0;

		float		fi = floor ( h );
		int		i  = int ( fi );
		float	f = h - fi;
		float	p = v * ( 1.0 - s );
		float	q = v * ( 1.0 - s * f );
		float	t = v * ( 1.0 - s * ( 1.0 - f ) );

		if ( i == 0 )
			res = vec3 ( v, t, p );
		else
		if ( i == 1 )
			res = vec3 ( q, v, p );
		else
		if ( i == 2 )
			res = vec3 ( p, v, t );
		else
		if ( i == 3 )
			res = vec3 ( p, q, v );
		else
		if ( i == 4 )
			res = vec3 ( t, p, v );
		else
			res = vec3 ( v, p, q );
	}

	return res;
}

void main (void)
{
	vec3	color = texture2D ( mainTex, gl_TexCoord [0].xy ).rgb;
	vec3	hsv   = rgbToHsv  ( color );

	hsv *= vec3 ( hScale, sScale, vScale );

	gl_FragColor = vec4 ( hsvToRgb ( hsv ), 1.0 );
}
