//
// Example of animated soap bubbles via GLSL programs
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include    "libExt.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include    <glut.h>
#endif

#include    <stdio.h>
#include    <stdlib.h>

#include    "libTexture.h"
#include    "TypeDefs.h"
#include    "Vector3D.h"
#include    "Vector2D.h"
#include    "boxes.h"
#include    "noise3D.h"
#include    "GlslProgram.h"

Vector3D    eye   ( -0.5, -0.5, 1.5 );  // camera position
unsigned    decalMap;                   // decal (diffuse) texture
unsigned    stoneMap;
unsigned    teapotMap;
unsigned	fringeMap;
unsigned	reflectionMap;
unsigned	noiseMap;
float       angle     = 0;
float       rot       = 0;
Vector3D	eta ( 1.05, 1.1, 1.15 );
Vector3D	r0;
bool        useFilter   = true;
bool		useRotation = true;

GlslProgram 	program;
NoiseTexture    noise;

void init ()
{
    glClearColor ( 0.0, 0.0, 0.0, 1.0 );
    glEnable     ( GL_DEPTH_TEST );
    glEnable     ( GL_TEXTURE_2D );
    glDepthFunc  ( GL_LEQUAL     );

    glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
    glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void displayBoxes ()
{
    glMatrixMode ( GL_MODELVIEW );
    glPushMatrix ();

    glActiveTextureARB ( GL_TEXTURE0_ARB );
    glEnable           ( GL_TEXTURE_2D   );
    glDisable          ( GL_TEXTURE_CUBE_MAP );

    glRotatef    ( rot, 0, 0, 1 );

    drawBox  ( Vector3D ( -5, -5, 0 ), Vector3D ( 10, 10, 3 ), stoneMap, false );
    drawBox  ( Vector3D ( 3, 2, 0.5 ), Vector3D ( 1,  2,  2 ), decalMap );

    glActiveTextureARB ( GL_TEXTURE1_ARB );
    glBindTexture      ( GL_TEXTURE_1D, fringeMap );

    glActiveTextureARB ( GL_TEXTURE2_ARB );
    glBindTexture      ( GL_TEXTURE_CUBE_MAP, reflectionMap );

    glActiveTextureARB ( GL_TEXTURE0_ARB );

	glEnable    ( GL_BLEND );
	glBlendFunc ( GL_SRC_ALPHA, GL_ONE );

    if ( useFilter )
        program.bind ();

    glBindTexture   ( GL_TEXTURE_2D, teapotMap );

    glPushMatrix ();
    glTranslatef ( 0.2, 1, 1.5 );

    glRotatef    ( angle * 45.3, 1, 0, 0 );
   	glRotatef    ( angle * 57.2, 0, 1, 0 );

    glutSolidTeapot ( 0.3 );

	glPopMatrix ();

    glPushMatrix ();
    glTranslatef ( 2.7, 2, 0.5 );

    glutSolidSphere ( 0.5, 20, 20 );

	glPopMatrix ();

    if ( useFilter )
        program.unbind ();

	glDisable   ( GL_BLEND );

    glActiveTextureARB ( GL_TEXTURE1_ARB );
    glBindTexture      ( GL_TEXTURE_3D, 0 );

    glActiveTextureARB ( GL_TEXTURE2_ARB );
    glBindTexture      ( GL_TEXTURE_1D, 0 );

    glActiveTextureARB ( GL_TEXTURE3_ARB );
    glBindTexture      ( GL_TEXTURE_CUBE_MAP, 0 );

    glPopMatrix ();
}

void display ()
{
    glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    displayBoxes ();

    glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
   gluLookAt      ( eye.x, eye.y, eye.z,    // eye
                    3, 3, 1,                // center
                    0, 0, 1 );              // up
}

void key ( unsigned char key, int x, int y )
{
    if ( key == 27 || key == 'q' || key == 'Q' )        // quit requested
        exit ( 0 );

    if ( key == ' ' )
    	useRotation = !useRotation;
}

void    specialKey ( int key, int x, int y )
{
    if ( key == GLUT_KEY_RIGHT )
        rot += 5;
    else
    if ( key == GLUT_KEY_LEFT )
        rot -= 5;

    glutPostRedisplay ();
}

void    animate ()
{
	static	float	lastTime = 0.0;
	float			time     = 0.001f * glutGet ( GLUT_ELAPSED_TIME );

    if ( useRotation )
	    angle += 2 * (time - lastTime);

    lastTime = time;

    program.bind ();
    program.setUniformVector ( "eyePos", eye );
    program.setUniformVector ( "r0",     r0  );
    program.setUniformFloat  ( "time",   time );
    program.unbind ();

    glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
                                // initialize glut
    glutInit            ( &argc, argv );
    glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    glutInitWindowSize  ( 512, 512 );


                                // create window
    glutCreateWindow ( "OpenGL soap bubbles" );

                                // register handlers
    glutDisplayFunc  ( display    );
    glutReshapeFunc  ( reshape    );
    glutKeyboardFunc ( key        );
    glutSpecialFunc  ( specialKey );
    glutIdleFunc     ( animate    );

    init           ();
    initExtensions ();

	if ( !GlslProgram :: isSupported () )
	{
		printf ( "GLSL not supported.\n" );

		return 1;
	}

    if ( !program.loadShaders ( "bubble.vsh", "bubble.fsh" ) )
    {
        printf ( "Error loading shaders:\n%s\n", program.getLog ().c_str () );

        return 3;
    }

	const char * faces [6] =
	{
		"../../Textures/Cubemaps/cm_left.tga",
		"../../Textures/Cubemaps/cm_right.tga",
		"../../Textures/Cubemaps/cm_top.tga",
		"../../Textures/Cubemaps/cm_bottom.tga",
		"../../Textures/Cubemaps/cm_back.tga",
		"../../Textures/Cubemaps/cm_front.tga",
	};

	reflectionMap = createCubeMap ( true, faces );

	fringeMap = createTexture1D ( true, "fringe.png" );

    decalMap  = createTexture2D ( true, "../../Textures/oak.bmp" );
    stoneMap  = createTexture2D ( true, "../../Textures/block.bmp" );
    teapotMap = createTexture2D ( true, "../../Textures/Oxidated.jpg" );
    noiseMap  = noise.createNoiseTexture3D ( 64, 64, 64 );

    program.bind ();
    program.setTexture ( "fringeMap",     1 );
    program.setTexture ( "reflectionMap", 2 );
    program.unbind ();

	r0.x = (1 - eta.x)/(1 + eta.x);
	r0.y = (1 - eta.y)/(1 + eta.y);
	r0.z = (1 - eta.z)/(1 + eta.z);
	r0  *= r0;

    printf ( "Space toggles rotation on/offs\n" );

    glutMainLoop ();

    return 0;
}
