//
// Example of procedural sky with clouds via GLSL programs
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include    "libExt.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include    <glut.h>
#endif

#include    <stdio.h>
#include    <stdlib.h>

#include    "libTexture.h"
#include    "libTexture3D.h"
#include    "TypeDefs.h"
#include    "Vector3D.h"
#include    "Vector2D.h"
#include	"Vector4D.h"
#include    "boxes.h"
#include    "GlslProgram.h"
#include    "Camera.h"

Vector3D    eye ( 0, 0, 0 );  			// camera position
Vector3D	sun ( 130, 130, 144 );			// sun position
unsigned    decalMap;                   // decal (diffuse) texture
unsigned    stoneMap;
unsigned    teapotMap;
unsigned    noiseMap;
unsigned	fringeMap;

float	yaw          = 0;
float	pitch        = 0;
float	roll         = 0;
float	cloudDensity = 0.5;

Camera			camera ( eye, 0, 0, 0 );	// camera to be used
GlslProgram 	program;

void init ()
{
    glClearColor ( 0.0, 0.0, 0.0, 1.0 );
    glEnable     ( GL_DEPTH_TEST );
    glEnable     ( GL_TEXTURE_2D );
    glDepthFunc  ( GL_LEQUAL     );

    glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
    glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void display ()
{
    glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	camera.apply ();

    glActiveTextureARB ( GL_TEXTURE1_ARB );
    glBindTexture      ( GL_TEXTURE_3D, noiseMap );

    program.bind ();

	glTranslatef ( 0, 0, -3 );

	glutSolidSphere ( 10, 20, 20 );

	glTranslatef ( 0, 0, 3 );

    program.unbind ();
/*
	glBegin ( GL_QUADS );
		glVertex3f ( -10, -10, -0.1 );
		glVertex3f ( -10,  10, -0.1 );
		glVertex3f (  10,  10, -0.1 );
		glVertex3f (  10, -10, -0.1 );
	glEnd ();
*/
    glutSwapBuffers ();
}

void reshape ( int w, int h )
{
	camera.setViewSize ( w, h, 60 );
	camera.apply       ();
}

void key ( unsigned char key, int x, int y )
{
    if ( key == 27 || key == 'q' || key == 'Q' )        // quit requested
        exit ( 0 );

    if ( key == '+' )
    	cloudDensity += 0.01;
    else
    if ( key == '-' )
    	cloudDensity -= 0.01;
}

void    specialKey ( int key, int x, int y )
{
    if ( key == GLUT_KEY_UP )
        yaw += M_PI / 90;
    else
    if ( key == GLUT_KEY_DOWN )
        yaw -= M_PI / 90;
	else
    if ( key == GLUT_KEY_RIGHT )
        roll += M_PI / 90;
    else
    if ( key == GLUT_KEY_LEFT )
        roll -= M_PI / 90;

	camera.setEulerAngles ( yaw, pitch, roll );

    glutPostRedisplay ();
}

void	mouseFunc ( int x, int y )
{
	static	int	lastX = -1;
	static	int	lastY = -1;

	if ( lastX == -1 )				// not initialized
	{
		lastX = x;
		lastY = y;
	}

	yaw  -= (y - lastY) * 0.02;
	roll += (x - lastX) * 0.02;

	lastX = x;
	lastY = y;

	camera.setEulerAngles ( yaw, pitch, roll );

	glutPostRedisplay ();
}

void    animate ()
{
	static	float	lastTime = 0.0;
	float			time     = 0.001f * glutGet ( GLUT_ELAPSED_TIME );

    lastTime = time;

    program.bind ();
    program.setUniformVector ( "eyePos",       Vector4D ( eye, 1 ) );
    program.setUniformVector ( "sunPos",       Vector4D ( sun, 1 ) );
    program.setUniformFloat  ( "time",         time );
    program.setUniformFloat  ( "cloudDensity", cloudDensity );
    program.unbind ();

    glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
                                // initialize glut
    glutInit            ( &argc, argv );
    glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    glutInitWindowSize  ( 512, 512 );


                                // create window
    glutCreateWindow ( "OpenGL procedural sky" );

                                // register handlers
    glutDisplayFunc       ( display    );
    glutReshapeFunc       ( reshape    );
    glutKeyboardFunc      ( key        );
    glutSpecialFunc       ( specialKey );
    glutPassiveMotionFunc ( mouseFunc  );
    glutIdleFunc          ( animate    );

    init           ();
    initExtensions ();

	if ( !GlslProgram :: isSupported () )
	{
		printf ( "GLSL not supported.\n" );

		return 1;
	}

    if ( !program.loadShaders ( "sky.vsh", "sky.fsh" ) )
    {
        printf ( "Error loading shaders:\n%s\n", program.getLog ().c_str () );

        return 3;
    }

	camera.setRightHanded ( false );

	noiseMap = createTexture3D ( false, "../../Textures/Noise/noise-3D.dds" );
    stoneMap  = createTexture2D ( true, "../../Textures/block.bmp" );

    program.bind ();
    program.setTexture ( "noiseMap", 1 );
    program.unbind ();

	printf ( "Use + and - keys to change cloud density" );

    glutMainLoop ();

    return 0;
}
