//
// Example of reflective water via GLSL programs
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include    "libExt.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include    <glut.h>
#endif

#include    <stdio.h>
#include    <stdlib.h>

#include    "libTexture.h"
#include    "libTexture3D.h"
#include    "TypeDefs.h"
#include	"Vector4D.h"
#include    "Vector3D.h"
#include    "Vector2D.h"
#include    "boxes.h"
#include    "GlslProgram.h"
#include    "Camera.h"

Vector3D    eye   ( 0, 0, 0 );  		// camera position
unsigned	noiseMap;
unsigned	reflectionMap;
float		yaw   = 0;
float		pitch = 0;
float		roll  = 0;
bool        useFilter   = true;
bool		useRotation = true;
int		 	object      = 0;
Vector3D	eta ( 1.05, 1.1, 1.15 );

GlslProgram 	program;
Camera			camera ( eye, yaw, pitch, roll );	// camera to be used

void init ()
{
    glClearColor ( 0.0, 0.0, 0.0, 1.0 );
    glEnable     ( GL_DEPTH_TEST );
    glEnable     ( GL_TEXTURE_2D );
    glDepthFunc  ( GL_LEQUAL     );

    glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
    glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void displayBoxes ()
{
    glMatrixMode ( GL_MODELVIEW );
    glPushMatrix ();

    glActiveTextureARB ( GL_TEXTURE1_ARB );
    glBindTexture      ( GL_TEXTURE_3D, noiseMap );

    glActiveTextureARB ( GL_TEXTURE0_ARB );

    drawCubicMappedBox ( Vector3D ( 4, 4, 4 ), reflectionMap );

    program.bind ();

	glPopMatrix  ();
    glPushMatrix ();

	drawQuad ( Vector3D ( -4, -4, -1 ), Vector3D ( 8, 0, 0 ), Vector3D ( 0, 8, 0 ) );

    program.unbind ();

    glPopMatrix ();
}

void display ()
{
    glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	camera.apply ();

	glMatrixMode ( GL_TEXTURE );
	glPushMatrix ();
	glRotatef    ( 90, 1, 0, 0 );

	glMatrixMode ( GL_MODELVIEW );

    displayBoxes ();

	glMatrixMode ( GL_TEXTURE );
	glPopMatrix  ();

	glMatrixMode ( GL_MODELVIEW );

    glutSwapBuffers ();
}

void reshape ( int w, int h )
{
	camera.setViewSize ( w, h, 60 );
	camera.apply       ();
}

void key ( unsigned char key, int x, int y )
{
    if ( key == 27 || key == 'q' || key == 'Q' )        // quit requested
        exit ( 0 );

   	if ( key == 'w' || key == 'W' )
   		camera.moveBy ( camera.getViewDir () * 0.2 );
   	else
   	if ( key == 'x' || key == 'X' )
   		camera.moveBy ( -camera.getViewDir () * 0.2 );
   	else
   	if ( key == 'a' || key == 'A' )
   		camera.moveBy ( -camera.getSideDir () * 0.2 );
   	else
   	if ( key == 'd' || key == 'D' )
   		camera.moveBy ( camera.getSideDir () * 0.2 );
}

void    specialKey ( int key, int x, int y )
{
    if ( key == GLUT_KEY_UP )
        yaw += M_PI / 90;
    else
    if ( key == GLUT_KEY_DOWN )
        yaw -= M_PI / 90;
	else
    if ( key == GLUT_KEY_RIGHT )
        roll += M_PI / 90;
    else
    if ( key == GLUT_KEY_LEFT )
        roll -= M_PI / 90;

	camera.setEulerAngles ( yaw, pitch, roll );

    glutPostRedisplay ();
}

void	mouseFunc ( int x, int y )
{
	static	int	lastX = -1;
	static	int	lastY = -1;

	if ( lastX == -1 )				// not initialized
	{
		lastX = x;
		lastY = y;
	}

	yaw  -= (y - lastY) * 0.02;
	roll += (x - lastX) * 0.02;

	lastX = x;
	lastY = y;

	camera.setEulerAngles ( yaw, pitch, roll );

	glutPostRedisplay ();
}

void    animate ()
{
	static	float	lastTime = 0.0;
	float			time     = 0.001f * glutGet ( GLUT_ELAPSED_TIME );

    lastTime = time;

    program.bind ();
    program.setUniformVector ( "eyePos", Vector4D ( eye, 1 ) );
    program.setUniformFloat  ( "time",   time );
    program.unbind ();

    glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
                                // initialize glut
    glutInit            ( &argc, argv );
    glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    glutInitWindowSize  ( 512, 512 );


                                // create window
    glutCreateWindow ( "OpenGL reflective water" );

                                // register handlers
    glutDisplayFunc  ( display    );
    glutReshapeFunc  ( reshape    );
    glutKeyboardFunc ( key        );
    glutSpecialFunc  ( specialKey );
    glutIdleFunc     ( animate    );
    glutPassiveMotionFunc ( mouseFunc );

    init           ();
    initExtensions ();

	if ( !GlslProgram :: isSupported () )
	{
		printf ( "GLSL not supported.\n" );

		return 1;
	}

    if ( !program.loadShaders ( "water.vsh", "water.fsh" ) )
    {
        printf ( "Error loading shaders:\n%s\n", program.getLog ().c_str () );

        return 3;
    }

	const char * faces [6] =
	{
		"../../Textures/Cubemaps/skyrt.bmp",
		"../../Textures/Cubemaps/skylf.bmp",
		"../../Textures/Cubemaps/skydn.bmp",
		"../../Textures/Cubemaps/skyup.bmp",
		"../../Textures/Cubemaps/skybk.bmp",
		"../../Textures/Cubemaps/skyfr.bmp",
	};

	noiseMap      = createTexture3D ( false, "../../Textures/Noise/noise-3D.dds" );
	reflectionMap = createCubeMap ( true, faces );
//	reflectionMap = createCubeMap ( true, "../../Textures/Cubemaps/Snow.dds" );

    program.bind ();
    program.setTexture ( "envMap",   0 );
    program.setTexture ( "noiseMap", 1 );
    program.unbind ();

	camera.setRightHanded ( false );

    glutMainLoop ();

    return 0;
}
