//
// Simple example of using Cg in OpenGL programs via CgProgram wrapper
//
// Author: Alex V. Boreskoff <steps3d@narod.ru>,<alexboreskoff@mtu-net.ru>
//

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include	<glut.h>
#endif

#include    <stdio.h>
#include    <stdlib.h>
#include	<Cg/cg.h>
#include    <Cg/cgGL.h>

#include    "libTexture.h"
#include    "TypeDefs.h"
#include    "Vector3D.h"
#include    "Vector2D.h"
#include	"Vector4D.h"
#include	"CgProgram.h"

/////////////////// here goes data for communication with Cg ///////////////////////

CgProgram	vertexProgram   ( CgProgram :: CgVertexProgram   );
CgProgram	fragmentProgram ( CgProgram :: CgFragmentProgram );

CGparameter	modelViewProjMatrix;	// handles for parameters
CGparameter modelViewMatrix;
CGparameter modelViewItMatrix;
CGparameter	lightPos, eyePos;

//////////////////////// here goes main program data ///////////////////////////////

Vector3D	eye   ( 7, 5, 7 );			// camera position
Vector3D	light ( 5, 0, 4 );			// light position
Vector3D	rot ( 0, 0, 0 );
float	 	angle     = 0;
int			mouseOldX = 0;
int			mouseOldY = 0;

unsigned	decalMap;

/////////////////////////// here goes main glut part ///////////////////////////////

void init ()
{
    glClearColor ( 0.0, 0.0, 0.0, 1.0 );
    glEnable     ( GL_DEPTH_TEST );
    glEnable     ( GL_TEXTURE_2D );
    glDepthFunc  ( GL_LEQUAL     );

    glHint       ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
    glHint       ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

									// now load cg programs
	if ( !vertexProgram.load ( "vertex.cg" ) )
	{
		printf ( "%s\n", vertexProgram.getErrorDescription ().c_str () );

		exit ( 1 );
	}

	fprintf ( stdout, "Vertex Listing:\n%s", vertexProgram.getListing ().c_str () );

	modelViewProjMatrix = vertexProgram.parameterForName ( "ModelViewProj" );
	modelViewMatrix     = vertexProgram.parameterForName ( "ModelView"     );
	modelViewItMatrix   = vertexProgram.parameterForName ( "ModelViewIT"   );
	lightPos            = vertexProgram.parameterForName ( "lightPos"      );
	eyePos              = vertexProgram.parameterForName ( "eyePos"        );

	if ( !fragmentProgram.load ( "fragment.cg" ) )
	{
		printf ( "%s\n", fragmentProgram.getErrorDescription ().c_str () );

		exit ( 1 );
	}

	fprintf ( stdout, "Fragment Listing:\n%s", fragmentProgram.getListing ().c_str () );
}

void display ()
{
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

												// draw the light
	glMatrixMode ( GL_MODELVIEW );
	glPushMatrix ();

	glDisable       ( GL_TEXTURE_2D );
	glTranslatef    ( light.x, light.y, light.z );
	glColor4f       ( 1, 1, 1, 1 );
	glutSolidSphere ( 0.1f, 15, 15 );

	glPopMatrix  ();
	glEnable     ( GL_TEXTURE_2D );

	glMatrixMode ( GL_MODELVIEW );
	glPushMatrix ();

    glRotatef    ( rot.x, 1, 0, 0 );
    glRotatef    ( rot.y, 0, 1, 0 );
    glRotatef    ( rot.z, 0, 0, 1 );

	vertexProgram.enable ();
	vertexProgram.bind   ();

	fragmentProgram.enable ();
	fragmentProgram.bind   ();

    cgGLSetStateMatrixParameter ( modelViewItMatrix,
                                  CG_GL_MODELVIEW_MATRIX, CG_GL_MATRIX_INVERSE_TRANSPOSE );

    cgGLSetStateMatrixParameter ( modelViewProjMatrix,
                                  CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY );

    cgGLSetStateMatrixParameter ( modelViewMatrix,
                                  CG_GL_MODELVIEW_MATRIX, CG_GL_MATRIX_IDENTITY );

	vertexProgram.setVector ( lightPos, Vector4D ( light ) );
	vertexProgram.setVector ( eyePos,   Vector4D ( eye   ) );

	glBindTexture  ( GL_TEXTURE_2D, decalMap );

	glutSolidTeapot ( 1.5 );

	vertexProgram  .disable ();
	fragmentProgram.disable ();

    glPopMatrix ();

	glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   											// factor all camera ops into projection matrix
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   gluLookAt      ( eye.x, eye.y, eye.z,	// eye
					0, 0, 0,				// center
					0, 0, 1 );				// up

   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
}

void motion ( int x, int y )
{
    rot.y -= ((mouseOldY - y) * 180.0f) / 200.0f;
    rot.z -= ((mouseOldX - x) * 180.0f) / 200.0f;
    rot.x  = 0;

    if ( rot.z > 360 )
		rot.z -= 360;

	if ( rot.z < -360 )
		rot.z += 360;

    if ( rot.y > 360 )
		rot.y -= 360;

	if ( rot.y < -360 )
        rot.y += 360;

    mouseOldX = x;
    mouseOldY = y;

	glutPostRedisplay ();
}

void mouse ( int button, int state, int x, int y )
{
    if ( state == GLUT_DOWN )
    {
        mouseOldX = x;
        mouseOldY = y;
	}
}

void key ( unsigned char key, int x, int y )
{
	if ( key == 27 || key == 'q' || key == 'Q' )	//	quit requested
    	exit ( 0 );
}

void	animate ()
{
	angle  = 0.004f * glutGet ( GLUT_ELAPSED_TIME );

	light.x = 4*cos ( angle );
	light.y = 4*sin ( angle );
	light.z = 3 + 0.3 * sin ( angle / 3 );

	vertexProgram.enable    ();
	vertexProgram.bind      ();
	vertexProgram.setVector ( lightPos, Vector4D ( light ) );
	vertexProgram.setVector ( eyePos,   Vector4D ( eye   ) );
	vertexProgram.disable   ();

	glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
								// initialize glut
	glutInit            ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize  ( 500, 500 );


								// create window
	glutCreateWindow ( "Example of Cg specular vertex shader via CgProgram wrapper class" );

								// register handlers
	glutDisplayFunc  ( display );
	glutReshapeFunc  ( reshape );
	glutKeyboardFunc ( key     );
	glutMouseFunc    ( mouse   );
	glutMotionFunc   ( motion  );
	glutIdleFunc     ( animate );

	init ();

	decalMap  = createTexture2D ( true, "../../Textures/wood1.bmp" );

    									// install program object as part of
    									// current state
	fragmentProgram.enable     ();
	fragmentProgram.bind       ();
	fragmentProgram.setTexture ( "tex0", decalMap  );
	fragmentProgram.disable    ();

    glutMainLoop ();

	return 0;
}
