//
// C++ wrapper for framebuffer object
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#ifndef	__FRAME_BUFFER__
#define	__FRAME_BUFFER__

#ifdef	_WIN32
	#include	<windows.h>
#endif

#ifdef	MACOSX
	#include	<OpenGL/gl.h>
#else
	#include	<GL/gl.h>
#endif

class	FrameBuffer
{
	int		flags;
	int		width;
	int		height;
	GLuint	frameBuffer;					// id of framebuffer object
	GLuint	colorBuffer [8];				// texture id or buffer id
	GLuint	depthBuffer;
	GLuint	stencilBuffer;					// 
	GLint	saveViewport [4];				// saved viewport setting during bind
	
public:
	FrameBuffer  ( int theWidth, int theHeight, int theFlags = 0 );
	~FrameBuffer ();
	
	int	getWidth () const
	{
		return width;
	}
	
	int	getHeight () const
	{
		return height;
	}
	
	bool	hasStencil () const
	{
		return stencilBuffer != 0;
	}
	
	bool	hasDepth () const
	{
		return depthBuffer != 0;
	}
	
	unsigned	getColorBuffer ( int no = 0 ) const
	{
		return colorBuffer [no];
	}

	unsigned	getDepthBuffer () const
	{
		return depthBuffer;
	}

	bool	isOk   () const;
	bool	create ();
	bool	bind   ();
	bool	unbind ();
	
	bool	attachColorTexture ( GLenum target, unsigned texId, int no = 0 );
	bool	attachDepthTexture ( GLenum target, unsigned texId );
	
	bool	detachColorTexture ( GLenum target )
	{
		return attachColorTexture ( target, 0 );
	}
	
	bool	detachDepthTexture ( GLenum target )
	{
		return attachDepthTexture ( target, 0 );
	}
	
										// create texture for attaching 
	unsigned	createColorTexture     ( GLenum format = GL_RGBA, GLenum internalFormat = GL_RGBA8, 
	                                     GLenum clamp = GL_REPEAT, int filter = filterMipmap );
										 
	unsigned	createColorRectTexture ( GLenum format = GL_RGBA, GLenum internalFormat = GL_RGBA8 );
	
										// mipmapping support
	void	buildMipmaps ( GLenum target = GL_TEXTURE_2D ) const
	{
		glBindTexture       ( target, getColorBuffer () );
		glGenerateMipmapEXT ( target );
	}
	
	enum								// flags for depth and stencil buffers
	{
		depth16 = 1,					// 16-bit depth buffer
		depth24 = 2,					// 24-bit depth buffer
		depth32 = 4,					// 32-bit depth buffer
		
		stencil1  = 16,					// 1-bit stencil buffer
		stencil4  = 32,					// 4-bit stencil buffer
		stencil8  = 64,					// 8-bit stencil buffer
		stencil16 = 128					// 16-bit stencil buffer
	};
	
	enum								// filter modes
	{
		filterNearest = 0,
		filterLinear  = 1,
		filterMipmap  = 2
	};
	
	static	bool	isSupported         ();
	static	int		maxColorAttachemnts ();
	static	int		maxSize             ();
};

#endif
