//--------------------------------------------------------------
//                     OpenGL Camera Control Demo
// Author: Roman Podobedov
// Email: romka@ut.ee
// WEB: http://romka.demonews.com
//--------------------------------------------------------------

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <direct.h>
#include <time.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "texture.h"
#include "camera.h"
#include "level.h"
#include "glf.h"

// Windows parameters
int winX = 0, winY = 0;
int winWidth = 1024, winHeight = 768;
char *className = "OpenGL";
char *windowName = "Flying Demo";
HDC hDC;
HDC hDCFrontBuffer;
HGLRC hGLRC;

unsigned *teximage;
int WindW, WindH;

GLuint texture[5]; /* Textures */

// Camera
CCamera camera;
CLevel level;

/* For FPS */
int frame_counter=0;
time_t curtime;
clock_t last_time;
float cur_time;
float total_frames;
char temp_fps[64];


static float gettime(void)
{
  clock_t time_new, time_raz;

  time_new=clock();
  time_raz=time_new-last_time;
  last_time=time_new;

  return(time_raz/(float)CLOCKS_PER_SEC);
}

/*=================== Load Texture =========================================*/
void LoadTexture(char *fn, int t_num)
{
	int texwid, texht;
	int texcomps;
	unsigned *teximage;
 
	teximage = read_texture(fn, &texwid, &texht, &texcomps);
	if (!teximage)
	{
		printf("Sorry, can't read texture file...");
		exit(0);
	}
	glBindTexture(GL_TEXTURE_2D, texture[t_num]);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, texwid, texht, 0, GL_RGBA, GL_UNSIGNED_BYTE, teximage);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, texwid, texht, GL_RGBA, GL_UNSIGNED_BYTE, teximage);

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR); 

	free(teximage);
} 

/*=================== Main Redraw Module ===================================*/
void Draw(HDC *hDC)
{
	float w, h, l;
	float x, y, z;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	camera.SetCamera();
	level.GetBounds(&w, &h, &l); // Get bounds of level cube
	camera.GetNewCoord(&x, &y, &z); // Get new coordinates from the camera
	if (((x > 1) && (x < w-1)) && ((y > 1) && (y < h-1)) && ((z > 1) && (z < l-1)))
		camera.Move(); // Move the camera, if inside the cube

	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glColor3f(1, 1, 1);
	
	level.Draw();

	/* Show FPS */

	frame_counter++;

	if((frame_counter % 40) == 0)
	{
		frame_counter = 0;
		cur_time=gettime();
		sprintf(temp_fps, "'a' - speed up, 'z' - speed down, FPS: %f\n", 40.0/cur_time);
	}

	glDisable(GL_CULL_FACE);
	glLoadIdentity();
	glTranslatef(-2, 1.45, -2);
	glScalef(0.06, 0.06, 1);
	glfDrawSolidString(temp_fps);
	glLoadIdentity();
	glTranslatef(0, 0, -2);
	glScalef(0.1, 0.1, 1);
	glfDrawSolidString("+");
	glEnable(GL_CULL_FACE);

	SwapBuffers(*hDC); /* Swap screen buffers */
}

void Init()
{
	srand((unsigned int)time(NULL));
	glClearColor(0, 0, 0, 0);
	camera.SetCameraPos(50, 50, 50);
	camera.SetCameraTargetPos(-10, -10, -10);
	level.LoadFromFile("levels/l1.dat");

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glEnable(GL_BLEND);

	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_DEPTH_TEST);

	glCullFace(GL_FRONT);
	glEnable(GL_CULL_FACE);

	glGenTextures(1, texture);
	LoadTexture("textures/cube.rgb", 0);
	glDisable(GL_TEXTURE_2D);

	glfInit();
	int result = glfLoadFont("fonts/times_new1.glf");
}

// Set projection
void setProjection(void)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(75, 1.333, 1, 3000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void resize(void)
{
    glViewport(0, 0, winWidth, winHeight);
    setProjection();
}

void setupPixelFormat(HDC hDC)
{
	PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	/* size of this pfd */
		1,				/* version num */
		PFD_DRAW_TO_WINDOW |
		PFD_DOUBLEBUFFER |
		PFD_SUPPORT_OPENGL,		/* support OpenGL */
		16,				/* pixel type */
		0,				/* 8-bit color depth */
		0, 0, 0, 0, 0, 0,		/* color bits (ignored) */
		0,				/* no alpha buffer */
		0,				/* alpha bits (ignored) */
		0,				/* no accumulation buffer */
		0, 0, 0, 0,			/* accum bits (ignored) */
		16,				/* depth buffer */
		0,				/* no stencil buffer */
		0,				/* no auxiliary buffers */
		PFD_MAIN_PLANE,			/* main layer */
		0,				/* reserved */
		0, 0, 0,			/* no layer, visible, damage masks */
	};

	int SelectedPixelFormat;
	BOOL retVal;

	SelectedPixelFormat = ChoosePixelFormat(hDC, &pfd);
	if (SelectedPixelFormat == 0)
	{
		(void) MessageBox(WindowFromDC(hDC),
		"Failed to find acceptable pixel format.",
		"OpenGL application error",
		MB_ICONERROR | MB_OK);
		exit(1);
    }

	retVal = SetPixelFormat(hDC, SelectedPixelFormat, &pfd);
	if (retVal != TRUE)
	{
		(void) MessageBox(WindowFromDC(hDC),
			"Failed to set pixel format.",
			"OpenGL application error",
			MB_ICONERROR | MB_OK);
		exit(1);
	}
}

LRESULT APIENTRY WndProc(
    HWND hWnd,
    UINT message,
    WPARAM wParam,
    LPARAM lParam)
{
	static UINT timer;
	static int omx, omy, mx, my;
  
	switch (message)
	{
		case WM_CREATE:
			timer = SetTimer(hWnd, 1, 1, NULL);
			return 0;
		case WM_DESTROY:
			//ChangeDisplaySettings(NULL,0);
			wglMakeCurrent(NULL, NULL);
			wglDeleteContext(hGLRC);
			KillTimer(hWnd, timer);
			PostQuitMessage(0);
			return 0;
		case WM_SIZE:
		if (hGLRC)
		{
			winWidth = (int) LOWORD(lParam);
			winHeight = (int) HIWORD(lParam);
			resize();
			return 0;
    		}
		case WM_PAINT:
	    if (hGLRC)
		{
			PAINTSTRUCT ps;
			BeginPaint(hWnd, &ps);
			Draw(&hDC);
  			EndPaint(hWnd, &ps);
			return 0;
	    }
	    break;

		case WM_TIMER:
			InvalidateRect(hWnd, NULL, FALSE);
			return 0;

	    case WM_MOUSEMOVE: /* Called when mouse is moved */
			mx = LOWORD(lParam);
			my = HIWORD(lParam);
			if  (mx & 1 << 15) mx -= (1 << 16);
			if  (my & 1 << 15) my -= (1 << 16);

			camera.SetMouse(mx, my, winWidth, winHeight);
			return 0;
		
		case WM_CHAR:
			switch ((int)wParam) 
			{
				case 'a':
					camera.IncSpeed(0.1);
					return 0;
				case 'z':
					camera.DecSpeed(0.1);
					return 0;
				case VK_ESCAPE: /* Esc is pressed */
					DestroyWindow(hWnd);
					return 0;
      			default:
			        break;
			}
	    break;
		default:
			break;
   }

  return DefWindowProc(hWnd, message, wParam, lParam);
}

/* Main Function */
int APIENTRY WinMain(
    HINSTANCE hCurrentInst,
    HINSTANCE hPreviousInst,
    LPSTR lpszCmdLine,
    int nCmdShow)
{
	WNDCLASS wndClass;
	HWND hWnd;
	DWORD dwExStyle;
	MSG msg;
	HDC hdc;
	DEVMODE md;
	int change;

	/* Get current screen resolution */
	hdc = GetDC(NULL);  
	winWidth = GetDeviceCaps(hdc, HORZRES);
	winHeight = GetDeviceCaps(hdc, VERTRES);

	wndClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wndClass.lpfnWndProc = (WNDPROC) WndProc;
	wndClass.cbClsExtra = 0;
	wndClass.cbWndExtra = 0;
	wndClass.hInstance = hCurrentInst;
	wndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndClass.hbrBackground = NULL;
	wndClass.lpszMenuName = NULL;
	wndClass.lpszClassName = className;
	RegisterClass(&wndClass);

	/* Change resolution of screen */

	memset(&md, 0, sizeof(md));
	md.dmSize       = sizeof(md);
	md.dmFields     = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
	md.dmPelsWidth  = winWidth;
	md.dmPelsHeight = winHeight;
	md.dmBitsPerPel = 16; /* Set 16 Bit per pixel */
	change = ChangeDisplaySettings(&md, CDS_FULLSCREEN); /* Change to fullscreen */


	dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;

	/* Create a window of the previously defined class */
	hWnd = CreateWindowEx(dwExStyle,
		className,		/* Window class's name */
		windowName,		/* Title bar text */
		/* The window's style */
		/*WS_OVERLAPPEDWINDOW*/WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
		winX, winY,		/* Position */
		winWidth, winHeight,	/* Size */
		NULL,			/* Parent window's handle */
		NULL,			/* Menu handle */
		hCurrentInst,		/* Instance handle */
		NULL);			/* No additional data */


	hDC = GetDC(hWnd);
	hDCFrontBuffer = hDC;
	setupPixelFormat(hDC);
	hGLRC = wglCreateContext(hDC);
	wglMakeCurrent(hDC, hGLRC);

	/* Map the window to the screen */
	ShowWindow(hWnd, nCmdShow);
	SetForegroundWindow(hWnd);
	SetFocus(hWnd);

	Init();
	setProjection();
	ShowCursor(FALSE);

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	ReleaseDC(hWnd, hDC);
	return msg.wParam;
}

