//--------------------------------------------------------------
//                     OpenGL Camera Control Demo
// Author: Roman Podobedov
// Email: romka@ut.ee
// WEB: http://romka.demonews.com
//--------------------------------------------------------------

#include <windows.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "camera.h"

const float pi = 3.14159f;
POINT p;

// Constructor
CCamera::CCamera()
{
	m_x = 1;
	m_y = 1;
	m_z = 1;
	m_tx = 0;
	m_ty = 0;
	m_tz = 0;
	angle_a = 0;
	angle_b = 0;
	last_angle_a = 0;
	last_angle_b = 0;
	speed_a = 0;
	speed_b = 0;
	m_speed = 0.2f;
	old_angle_a = 0;
	old_angle_b = 0;
}

CCamera::~CCamera()
{


}

// Set camera position
void CCamera::SetCameraPos(float x, float y, float z)
{
	m_x = x;
	m_y = y;
	m_z = z;
}

void CCamera::SetCameraTargetPos(float x, float y, float z)
{
	m_tx = x;
	m_ty = y;
	m_tz = z;
}

int sgn(float a)
{
	if (a > 0) return 1;
	if (a < 0) return -1;
	return 0;
}

// Get coordinates from the mouse
void CCamera::SetMouse(int mx, int my, int WinWidth, int WinHeight)
{

	angle_b = ((float)mx - (float)WinWidth/2)/((float)WinWidth/4);
	angle_a = ((float)WinHeight/2 - (float)my)/((float)WinHeight/3);

	GetCursorPos(&p);
	
	if (p.x > WinWidth/2) p.x -= 2;
	else if (p.x < WinWidth/2) p.x += 2;

	if (p.y > WinHeight/2) p.y -= 2;
	else if (p.y < WinHeight/2) p.y += 2;
}

// Increase speed of camera
void CCamera::IncSpeed(float speed)
{
	m_speed += speed;	
	if (m_speed > 5) m_speed = 5;
}

// Decrease speed of camera
void CCamera::DecSpeed(float speed)
{
	m_speed -= speed;
	if (m_speed < -5) m_speed = -5;
}

void CCamera::GetNewCoord(float *x, float *y, float *z)
{
	*x = new_x;
	*y = new_y;
	*z = new_z;
}

// Move the camera
void CCamera::Move()
{
	m_x = new_x;
	m_y = new_y;
	m_z = new_z;
}

// Set camera matrix in OpenGL
void CCamera::SetCamera()
{
	float phi, teta;
	float x2, y2, z2;

	phi = ((-old_angle_a * pi)/180);
	teta = ((old_angle_b * pi)/180);

	x2 = -cos(phi)*sin(teta);
	y2 = -sin(phi);
	z2 = cos(phi)*cos(teta);

	// First, rotate camera
	glRotatef(-old_angle_a, 1, 0, 0);
	glRotatef(old_angle_b, 0, 1, 0);
	// ... and set gluLookAt
	gluLookAt(m_x, m_y, m_z, m_x, m_y, m_z+1, 0, 1, 0);

	// Calculate new coordinates of camera
	new_x = m_x + x2*m_speed;
	new_y = m_y + y2*m_speed;
	new_z = m_z + z2*m_speed;

	old_angle_a += angle_a;
	old_angle_b += angle_b;

	if (old_angle_a > 359) old_angle_a = 360 - old_angle_a;
	if (old_angle_b > 359) old_angle_b = 360 - old_angle_b;

	SetCursorPos(p.x, p.y);
}

