//--------------------------------------------------------------
//                     OpenGL Camera Control Demo
// Author: Roman Podobedov
// Email: romka@ut.ee
// WEB: http://romka.demonews.com
//--------------------------------------------------------------

#include <windows.h>
#include <stdio.h>
#include <GL/gl.h>
#include "level.h"

// String normalization (remove all unused characters)
static char *NormalizeString(char *s)
{
	while (*s == ' ') s++;

	ULONG l = strlen(s);
	for (ULONG i=l; i>0; i--)
		if (s[i-1] < 33) s[i-1] = 0;
		else break;

	return s;
}

static void ParseFloat(char *s, float *a, float *b, float *c)
{
	int i, j;
	int x, y, z;
  
	s = NormalizeString(s);
	x = 0; y = -1; z = -1;
	j = (int)strlen(s);
	for (i=0; i<j; i++)
	{
		if ((s[i] == ' ') || (s[i] == '\n'))
		{
			s[i] = 0;
			if (y == -1) y = i+1;
			else
			if (z == -1) z = i+1;
		}
	}
	*a = (float)atof(&s[x]);
	*b = (float)atof(&s[y]);
	*c = (float)atof(&s[z]);
}

static void ParseFloat4(char *s, float *a, float *b, float *c, float *d)
{
	int i, j;
	int x, y, z, q;
  
	s = NormalizeString(s);
	x = 0; y = -1; z = -1, q = -1;
	j = (int)strlen(s);
	for (i=0; i<j; i++)
	{
		if ((s[i] == ' ') || (s[i] == '\n'))
		{
			s[i] = 0;
			if (y == -1) y = i+1;
			else
			if (z == -1) z = i+1;
			else
			if (q == -1) q = i+1;
		}
	}
	*a = (float)atof(&s[x]);
	*b = (float)atof(&s[y]);
	*c = (float)atof(&s[z]);
	*d = (float)atof(&s[q]);
}

CCube::CCube()
{
	next = NULL;
}

CCube::~CCube()
{
	if (next) delete next;
}

// Draw cube
void CCube::Draw()
{
	glColor3f(1, 1, 1);
	glBegin(GL_QUADS);
		
		glTexCoord2f(0, 0); glVertex3f(x, y, z);
		glTexCoord2f(1, 0); glVertex3f(x+lenght, y, z);
		glTexCoord2f(1, 1); glVertex3f(x+lenght, y+lenght, z);
		glTexCoord2f(0, 1); glVertex3f(x, y+lenght, z);

		glTexCoord2f(0, 0); glVertex3f(x, y, z+lenght);
		glTexCoord2f(1, 0); glVertex3f(x, y+lenght, z+lenght);
		glTexCoord2f(1, 1); glVertex3f(x+lenght, y+lenght, z+lenght);
		glTexCoord2f(0, 1); glVertex3f(x+lenght, y, z+lenght);

		glTexCoord2f(0, 0); glVertex3f(x, y, z);
		glTexCoord2f(1, 0); glVertex3f(x, y+lenght, z);
		glTexCoord2f(1, 1); glVertex3f(x, y+lenght, z+lenght);
		glTexCoord2f(0, 1); glVertex3f(x, y, z+lenght);

		glTexCoord2f(0, 0); glVertex3f(x+lenght, y, z);
		glTexCoord2f(1, 0); glVertex3f(x+lenght, y, z+lenght);
		glTexCoord2f(1, 1); glVertex3f(x+lenght, y+lenght, z+lenght);
		glTexCoord2f(0, 1); glVertex3f(x+lenght, y+lenght, z);

		glTexCoord2f(0, 0); glVertex3f(x, y, z);
		glTexCoord2f(1, 0); glVertex3f(x, y, z+lenght);
		glTexCoord2f(1, 1); glVertex3f(x+lenght, y, z+lenght);
		glTexCoord2f(0, 1); glVertex3f(x+lenght, y, z);

		glTexCoord2f(0, 0); glVertex3f(x, y+lenght, z);
		glTexCoord2f(1, 0); glVertex3f(x+lenght, y+lenght, z);
		glTexCoord2f(1, 1); glVertex3f(x+lenght, y+lenght, z+lenght);
		glTexCoord2f(0, 1); glVertex3f(x, y+lenght, z+lenght);

	glEnd();
	
	if (next) next->Draw();
}

CLevel::CLevel()
{
	cubes = NULL;
}

CLevel::~CLevel()
{
	delete cubes;
}

// Load level from file
void CLevel::LoadFromFile(char *file_name)
{
	FILE *f;
	char buffer[512];
	char *str;
	char flag=0;
	float xx, yy, zz, ll;
	CCube *temp, *cur;

	f = fopen(file_name, "rb");

	while (!feof(f))
	{
		fgets(buffer, 512, f);

		str = NormalizeString(&buffer[0]);
		
		if ((str[0] == '#') || (str[0] == 0)) continue;

		if (!flag)
		{
			ParseFloat(str, &m_width, &m_height, &m_lenght);
			flag = 1;
		}
		else
		{
			ParseFloat4(str, &xx, &yy, &zz, &ll);
			if (!cubes)
			{
				cubes = new CCube;
				cur = cubes;
			}
			else
			{
				temp = new CCube;
				cur->next = temp;
				cur = temp;
			}
			cur->next = NULL;
			cur->x = xx;
			cur->y = yy;
			cur->z = zz;
			cur->lenght = ll;
		}
		str[0] = 0;
	}

	fclose(f);
}

// Draw level
void CLevel::Draw()
{
	const int step = 10;
	float x, y, z;
	int i;

	glColor4f(0, 1, 0, 1);
	glBegin(GL_LINES);

	x = 0; y = 0; z = 0;
	for (i=0; i<=step; i++)
	{
		glVertex3f(x, 0, 0);
		glVertex3f(x, m_height, 0);

		glVertex3f(x, 0, m_lenght);
		glVertex3f(x, m_height, m_lenght);
		
		glVertex3f(0, 0, z);
		glVertex3f(0, m_height, z);

		glVertex3f(m_width, 0, z);
		glVertex3f(m_width, m_height, z);



		glVertex3f(0, y, 0);
		glVertex3f(m_width, y, 0);

		glVertex3f(0, y, m_lenght);
		glVertex3f(m_width, y, m_lenght);

		glVertex3f(0, y, 0);
		glVertex3f(0, y, m_lenght);

		glVertex3f(m_width, y, 0);
		glVertex3f(m_width, y, m_lenght);

		
		x += m_width / step;
		y += m_height / step;
		z += m_lenght / step;
	}
	glEnd();

	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);

	// Enable textures and draw cubes
	cubes->Draw();
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
}

// Get bounds of main cube
void CLevel::GetBounds(float *w, float *h, float *l)
{
	*w = m_width;
	*h = m_height;
	*l = m_lenght;
}

